package com.upyuns.platform.rs.datacenter.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.github.wxiaoqi.security.common.exception.BaseException;
import com.upyuns.platform.rs.datacenter.entity.RscpAreaImageTotal;
import com.upyuns.platform.rs.datacenter.entity.RscpAreaInfo;
import com.upyuns.platform.rs.datacenter.entity.RscpImageDataTotal;
import com.upyuns.platform.rs.datacenter.entity.RscpImageInputLog;
import com.upyuns.platform.rs.datacenter.fegin.RscloudboxClientRest;
import com.upyuns.platform.rs.datacenter.mapper.RscpAreaInfoMapper;
import com.upyuns.platform.rs.datacenter.mapper.RscpImageDataTotalMapper;
import com.upyuns.platform.rs.datacenter.pojo.ImageTotalInputDTO;
import com.upyuns.platform.rs.datacenter.pojo.imagedata.BgImageQueryDTO;
import com.upyuns.platform.rs.datacenter.pojo.imagedata.QueryDTO;
import com.upyuns.platform.rs.datacenter.pojo.rscloudbox.ImageDataDto;
import com.upyuns.platform.rs.datacenter.rest.RscpImageDataTotalController;
import com.upyuns.platform.rs.datacenter.vo.ImageInputVO;
import com.upyuns.platform.rs.datacenter.vo.MaxMinVO;
import com.upyuns.platform.rs.gtdata.GtDataRestClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static com.github.wxiaoqi.security.common.util.process.ResultCode.PARAM_ILLEGAL_CODE;
import static com.upyuns.platform.rs.datacenter.constant.DataConstant.THUMBNAIL_PATH;

@Service
@Slf4j
public class RscpImageDataTotalBiz extends BaseBiz<RscpImageDataTotalMapper, RscpImageDataTotal> {

    @Autowired
    RscpImageInputLogBiz rscpImageInputLogBiz;

    @Autowired
    RscpImageDataTotalBiz rscpImageDataTotalBiz;

    @Autowired
    RscpAreaImageTotalBiz rscpAreaImageTotalBiz;

    @Autowired
    RscpAreaInfoMapper rscpAreaInfoMapper;

    @Autowired
    RscloudboxClientRest rscloudboxClientRest;

    @Autowired
    GtDataRestClient gtDataRestClient;

    public void inStorage(ImageInputVO ii) {
        ExecutorService executorService = Executors.newCachedThreadPool();
        executorService.execute(() -> {
            ii.setThumbnailPath(ii.getThumbnailPath().replace(THUMBNAIL_PATH, ""));
            RscpImageDataTotal rscpImageDataTotal= null;
            try {
                //查询数据是否存在
                rscpImageDataTotal = rscpImageDataTotalBiz.selectOne(new RscpImageDataTotal(){{
                    setName(ii.getName());
                }});

                ImageTotalInputDTO imageTotalInputDTO = BeanUtil.toBean(ii, ImageTotalInputDTO.class);

                if(null != rscpImageDataTotal) {
                    //更新
                    BeanUtil.copyProperties(ii, rscpImageDataTotal);
//                    rscpImageDataTotal.setImageResolution(null);
                    rscpImageDataTotalBiz.updateSelectiveById(rscpImageDataTotal);
                }else {
                    rscpImageDataTotal = BeanUtil.toBean(ii, RscpImageDataTotal.class);
                    rscpImageDataTotalBiz.insertSelective(rscpImageDataTotal);
                    RscpImageDataTotal finalRscpImageDataTotal = rscpImageDataTotal;
                    rscpImageDataTotalBiz.updateSelectiveById(new RscpImageDataTotal(){{
                        setId(finalRscpImageDataTotal.getId());
                        setDataId(finalRscpImageDataTotal.getId());
                    }});
                }
                imageTotalInputDTO.setId(rscpImageDataTotal.getId());
                //更新几何范围
                updateGeom(imageTotalInputDTO);

                //更新地区信息
                rscpAreaImageTotalBiz.inStorageAreaDeal(imageTotalInputDTO);

                //插入接收系统
                inputbox(imageTotalInputDTO.getId());

            }catch (Exception e) {
                log.error(e.getMessage(), e);
                if(null != ii.getLogId()) {
                    rscpImageInputLogBiz.updateSelectiveById(new RscpImageInputLog(){{
                        setId(ii.getLogId());
                        setStatus(STATU_FAIL);
                        setResult(e.getMessage());
                    }});
                }
                throw e;
            }finally {
                if(null != ii.getLogId()) {
                    if(null != rscpImageDataTotal) {
                        String message = rscpImageDataTotal.getId()+ "";
                        rscpImageInputLogBiz.updateSelectiveById(new RscpImageInputLog(){{
                            setId(ii.getLogId());
                            setStatus(STATU_SUCC);
                            setResult(message);
                            //setResult(JSONUtil.toJsonStr(CollUtil.newArrayList(resultDTO)));
                        }});

                    }else {
                        rscpImageInputLogBiz.updateSelectiveById(new RscpImageInputLog(){{
                            setId(ii.getLogId());
                            setStatus(STATU_FAIL);
                        }});
                    }
                }
            }
        });
    }

    public List<RscpImageDataTotal> queryDataList(QueryDTO dto) {
        return mapper.queryDataList(dto);
    }

    public List<RscpImageDataTotal> bgQueryDataList(BgImageQueryDTO dto) {
        return mapper.bgQueryDataList(dto);
    }


    public RscpImageDataTotal queryDataList(Long id) {
        List<RscpImageDataTotal> list2 = mapper.queryDataById(id);
        if(list2.size() > 0) {
//            RscpImageDataTotal rscpImageDataTotal = list2.get(0);
            return list2.get(0);
        }
        throw new BaseException("数据不存在", PARAM_ILLEGAL_CODE);
    }

    public List<RscpAreaImageTotal> queryDataAreaList(QueryDTO dto) {
        List<RscpImageDataTotal> list = mapper.queryDataList(dto);
        List<RscpAreaImageTotal> list2 = CollUtil.newArrayList();
        if(CollUtil.isNotEmpty(list)) {
//            for(RscpImageDataTotal ridt : list) {
//                RscpAreaImageTotal rait = BeanUtil.toBean(ridt, RscpAreaImageTotal.class);
//                rait.setBeginTime(ridt.getImageTakeTime());
//                rait.setImageResolution(rait.getImageResolution().replace("[","{").replace("]", "}"));
//                list2.add(rait);
//            }
            list2 = Convert.toList(RscpAreaImageTotal.class, list);
        }
        return list2;
    }

    public String getBinNameByGId(ImageTotalInputDTO dto){
//        String param = StrUtil.format("MULTIPOLYGON((({} {}  ,{} {}  , {} {}  ,#{rightTopLon} #{rightTopLat},   #{leftTopLon} #{leftTopLat})))");
        return mapper.getBinNameByGId(dto.getLeftTopLon(), dto.getLeftTopLat(), dto.getLeftBottomLon(), dto.getLeftBottomLat(), dto.getRightBottomLon(), dto.getRightBottomLat(), dto.getRightTopLon(), dto.getRightTopLat());
    }

    public Integer updateGeom(ImageTotalInputDTO dto){
        return mapper.updateGeom(dto.getLeftTopLon(), dto.getLeftTopLat(), dto.getLeftBottomLon(), dto.getLeftBottomLat(), dto.getRightBottomLon(), dto.getRightBottomLat(), dto.getRightTopLon(), dto.getRightTopLat(), dto.getId());
    }

    public MaxMinVO coverStatisticsMaxMin() {
        return mapper.coverStatisticsMaxMin();
    }

    public String geoJsonByRange(String rangeMeta){
        return mapper.geoJsonByRange(rangeMeta);
    }

    public void aaa(ImageInputVO vo) {
        ImageTotalInputDTO dto = BeanUtil.toBean(vo, ImageTotalInputDTO.class);
//        dto.setRange(getBinNameByGId(dto));
        List<RscpAreaInfo> imageAreaInfoList = rscpAreaInfoMapper.getIntersectAreaInfo(dto.getLeftTopLon(), dto.getLeftTopLat(), dto.getLeftBottomLon(), dto.getLeftBottomLat(), dto.getRightBottomLon(), dto.getRightBottomLat(), dto.getRightTopLon(), dto.getRightTopLat());
    }

    public String queryGeomsJson(QueryDTO dto){
        return mapper.queryGeomsJson(dto);
    }

    public Map<String, Object>  inputbox(Long dataid) {
        RscpImageDataTotal rscpImageDataTotal = queryDataList(dataid);
        ImageDataDto imageDataDto = new ImageDataDto();
        imageDataDto.setId(rscpImageDataTotal.getId().toString());
//        imageDataDto.setAreaDescription(null);
//        imageDataDto.setAreaNo(null);
//        imageDataDto.setAreaNoArray(null);
//        imageDataDto.setAutoTag(null);
        imageDataDto.setImageTakeTime(rscpImageDataTotal.getImageTakeTime());
        imageDataDto.setImageEndTime(rscpImageDataTotal.getImageEndTime());
        imageDataDto.setImageStartTime(rscpImageDataTotal.getImageStartTime());
        imageDataDto.setImageProductTime(rscpImageDataTotal.getImageProductTime());
//        imageDataDto.setDataId(rscpImageDataTotal.getId().toString());
//        imageDataDto.setGemo(null);
        imageDataDto.setImageArea(StrUtil.isNotBlank(rscpImageDataTotal.getImageArea()) ? rscpImageDataTotal.getImageArea() : null);
        imageDataDto.setImageCity(rscpImageDataTotal.getImageCity());
        imageDataDto.setImageCloudage(null != rscpImageDataTotal.getImageCloudage() ? rscpImageDataTotal.getImageCloudage().doubleValue() : null);
//        Double resolution = (rscpImageDataTotal.getImageResolution() == null || rscpImageDataTotal.getImageResolution().length <= 0)?
//                null: rscpImageDataTotal.getImageResolution()[0].doubleValue();
//        rscmAreaImageDto.setImageEndResolution(resolution);

        if((rscpImageDataTotal.getImageResolution() == null || rscpImageDataTotal.getImageResolution().length <= 0)) {
            imageDataDto.setImageResolution("");
        }else {
            String resolutionStr = "{";
            for(int i = 0; i < rscpImageDataTotal.getImageResolution().length; i++) {
                BigDecimal resolution = rscpImageDataTotal.getImageResolution()[i];
                resolutionStr += resolution;
                if(i == rscpImageDataTotal.getImageResolution().length - 1) {
                    resolutionStr += "}";
                }else {
                    resolutionStr += ",";
                }
//                if(i = 0) {
//                    rscmAreaImageDto.setImageStartResolution();
//                }
            }
            imageDataDto.setImageResolution(resolutionStr);
        }

        imageDataDto.setImageIsSave(1);
        imageDataDto.setImageProductType(rscpImageDataTotal.getImageProductType());
        imageDataDto.setImageProjection(rscpImageDataTotal.getImageProjection());
        imageDataDto.setImageRowCol(rscpImageDataTotal.getImageRowCol());
        imageDataDto.setImageSatelliteType(rscpImageDataTotal.getImageSatelliteType());
        imageDataDto.setImageSensorType(rscpImageDataTotal.getImageSensorType());
//        imageDataDto.setImageSize(null);
//        rscmAreaImageDto.setImageSpectrumType(null != rscpImageDataTotal.getImageSpectrumType() ? rscpImageDataTotal.getImageSpectrumType().toString() : null);
        if(null == rscpImageDataTotal.getImageSpectrumType() || rscpImageDataTotal.getImageSpectrumType().length <= 0) {
            imageDataDto.setImageSpectrumType("");
        }else {
            String spectrumTypeStr = "{";
            for(int i = 0; i < rscpImageDataTotal.getImageSpectrumType().length; i++) {
                String pectrumType = rscpImageDataTotal.getImageSpectrumType()[i];
                spectrumTypeStr += pectrumType;
                if(i == rscpImageDataTotal.getImageSpectrumType().length - 1) {
                    spectrumTypeStr += "}";
                }else {
                    spectrumTypeStr += ",";
                }
            }
            imageDataDto.setImageSpectrumType(spectrumTypeStr);
        }

        if(null == rscpImageDataTotal.getImageSpectrumTypeDisplay() || rscpImageDataTotal.getImageSpectrumTypeDisplay().length <= 0) {
            imageDataDto.setImageSpectrumTypeDisplay("");
        }else {
            String spectrumTypeStr = "{";
            for(int i = 0; i < rscpImageDataTotal.getImageSpectrumTypeDisplay().length; i++) {
                String pectrumType = rscpImageDataTotal.getImageSpectrumTypeDisplay()[i];
                spectrumTypeStr += pectrumType;
                if(i == rscpImageDataTotal.getImageSpectrumTypeDisplay().length - 1) {
                    spectrumTypeStr += "}";
                }else {
                    spectrumTypeStr += ",";
                }
            }
            imageDataDto.setImageSpectrumTypeDisplay(spectrumTypeStr);
        }

//        imageDataDto.setImgUrl(null);
        imageDataDto.setCorrectStatus(rscpImageDataTotal.getCorrectStatus());
//        imageDataDto.setIsCover(null);
//        imageDataDto.setJobid(null);
        imageDataDto.setName(rscpImageDataTotal.getName());
//        imageDataDto.setNum(null);
        imageDataDto.setImageProductId(rscpImageDataTotal.getImageProductId());
        imageDataDto.setImageProductLevel(rscpImageDataTotal.getImageProductLevel());
        imageDataDto.setRange(rscpImageDataTotal.getRange());
//        imageDataDto.setRecordId(null);
//        imageDataDto.setRelationNo(null);
        imageDataDto.setImageSensorId(rscpImageDataTotal.getImageSensorId());
//        imageDataDto.setSrid(null);
//        imageDataDto.setTip(null);
//        rscmAreaImageDto.setUpdateTime(rscpImageDataTotal.getCreateDt());

        imageDataDto.setImageSceneId(rscpImageDataTotal.getImageSceneId());

        imageDataDto.setImageFilePath(rscpImageDataTotal.getImageFilePath());
        imageDataDto.setThumbnailPath(rscpImageDataTotal.getThumbnailPath());

        //复制到zmp目录
        String sourcePath = rscpImageDataTotal.getImageFilePath();
        Integer index = rscpImageDataTotal.getImageFilePath().lastIndexOf(rscpImageDataTotal.getName()+".tar.gz");
        if(index > 0) {
            sourcePath = rscpImageDataTotal.getImageFilePath().substring(0, index);
        }

        if(sourcePath.startsWith("/")) {
            sourcePath = sourcePath.substring(1);
        }

        String targetPath = sourcePath;
        targetPath = targetPath.replaceAll("rscloudmart", "zmp");
        Map<String, Object> copy0Map = gtDataRestClient.mkdirs(targetPath);
        Boolean sign = false;
        if (200 == (Integer) copy0Map.get("HttpStatusCode") ) {
            sign = true;
        }else if(2002 == (Integer) ((Map<String, Object>) copy0Map.get("GTDataException")).get("errorCode")) {
            sign = true;
        }else {
            log.warn(MessageFormat
                    .format("mkdir ({0}) ... Fail!Code: {1} Message: {2}",
                            targetPath,
                            copy0Map.get("result"),
                            copy0Map.get("message")));
            copy0Map.put("targetPath", targetPath);
//            return copy0Map;
        }
        if(sign) {
            String sourceFile = sourcePath + rscpImageDataTotal.getName()+".tar.gz";
            String targetFile = targetPath + rscpImageDataTotal.getName()+".tar.gz";
            Map<String, Object> copyMap = gtDataRestClient.copy(sourceFile, targetFile);
            if (200 != (Integer) copyMap.get("HttpStatusCode") && 2002 != (Integer) ((Map<String, Object>) copy0Map.get("GTDataException")).get("errorCode")) {
                log.warn(MessageFormat
                        .format("Copy data file({0}) to {1}... Fail!Code: {2} Message: {3}",
                                sourcePath,
                                targetPath,
                                copyMap.get("result"),
                                copyMap.get("message")));
                copyMap.put("sourcePath", sourcePath);
                copyMap.put("targetPath", targetPath);
//                return copyMap;
            }else {
                //复制压缩图
                String sourceTPath = rscpImageDataTotal.getThumbnailPath();
                sourceTPath = THUMBNAIL_PATH + sourceTPath + "_800_800.png";
                String targetTPath = targetPath + rscpImageDataTotal.getName()+ "_800_800.png";
                Map<String, Object> copy2Map = gtDataRestClient.copy(sourceTPath, targetTPath);
                if (200 != (Integer) copyMap.get("HttpStatusCode") ) {
                    log.warn(MessageFormat
                            .format("Copy data file({0}) to {1}... Fail!Code: {2} Message: {3}",
                                    sourcePath,
                                    targetPath,
                                    copyMap.get("result"),
                                    copyMap.get("message")));
                    copyMap.put("sourceTPath", sourcePath);
                    copyMap.put("targetTPath", targetPath);
//                    return copy2Map;
                }
            }
        }


        return
//                CollUtil.newHashMap();
                rscloudboxClientRest.saveImage(imageDataDto);
    }

    public static void main(String[] args) {
        String name = "GF6_PMS_E126.5_N46.8_20230903_L1A1420349421";
        String path = "/rscloudmart/data/GF6//GF6_PMS_E126.5_N46.8_20230903_L1A1420349421//GF6_PMS_E126.5_N46.8_20230903_L1A1420349421.tar.gz";
        Integer index = path.lastIndexOf(name+".tar.gz");
        if(path.lastIndexOf(name+".tar.gz") > 0) {
            String sourcePath = path.substring(0, index);
            System.out.println(sourcePath);
        }
    }
}
